@@ -70,8 +70,8 @@ from distutils import sysconfig
 
 
 #  Download details for the latest PySide release.
-PYSIDE_SOURCE_MD5 = "71869b10d0562d8f5a2761d2fb298677"
-PYSIDE_SOURCE_URL = "http://www.pyside.org/files/pyside-qt4.7+1.0.0~beta2.tar.bz2"
+PYSIDE_SOURCE_MD5 = "d6d7eae6744875fcf9862a827ab6c4aa"
+PYSIDE_SOURCE_URL = "http://www.pyside.org/files/pyside-qt4.7+1.0.0~beta5.tar.bz2"
 
 
 #  Classes that must not be hacked out of the PySide binary.
@@ -98,6 +98,7 @@ KEEP_METHODS = {
          )),
     "QBitArray": set(("setBit",)),
     "QByteArray": set(("insert",)),
+    "QFileDialog": set(("*",)),
 }
 
 
@@ -154,6 +155,9 @@ class Hatchet(object):
     will be used.
     """
 
+    SOURCE_URL = PYSIDE_SOURCE_URL
+    SOURCE_MD5 = PYSIDE_SOURCE_MD5
+
     def __init__(self,appdir,mf=None,typedb=None,logger=None):
         self.appdir = appdir
         if mf is None:
@@ -304,11 +308,12 @@ class Hatchet(object):
                     except (zipfile.BadZipfile,):
                         pass
                 else:
-                    try:
-                        if "executable" in _bt("file",subpath):
-                            self.add_zipfile(subpath,**rkwds)
-                    except (EnvironmentError,zipfile.BadZipfile,):
-                        pass
+                    if sys.platform != "win32":
+                        try:
+                            if "executable" in _bt("file",subpath):
+                                self.add_zipfile(subpath,**rkwds)
+                        except (EnvironmentError,zipfile.BadZipfile,):
+                            pass
 
     def expand_kept_classes(self):
         """Find classes and methods that might be used by the application.
@@ -325,17 +330,22 @@ class Hatchet(object):
         used_ids = set()
         for m in self.mf.modules.itervalues():
             if m.__code__ is not None:
+                self.logger.debug("examining code: %s",m)
                 self.find_identifiers_in_code(m.__code__,used_ids)
         #  Keep all classes used directly in code.
         for classnm in self.typedb.iterclasses():
-            if classnm in used_ids or classnm in KEEP_CLASSES:
-                self.logger.debug("keeping class: %s",classnm)
+            if classnm in used_ids:
+                self.logger.debug("keeping class: %s [used]",classnm)
+                self.keep_classes.add(classnm)
+            if classnm in KEEP_CLASSES:
+                self.logger.debug("keeping class: %s [pinned]",classnm)
                 self.keep_classes.add(classnm)
         #  Keep all superclasses of all kept classes
         for classnm in list(self.keep_classes):
             for sclassnm in self.typedb.superclasses(classnm):
                 if sclassnm not in self.keep_classes:
-                    self.logger.debug("keeping class: %s",sclassnm)
+                    msg = "keeping class: %s [sup %s]"
+                    self.logger.debug(msg,sclassnm,classnm)
                     self.keep_classes.add(sclassnm)
         #  Now iteratively expand the kept classess with possible return types
         #  of any methods called on the kept classes.
@@ -354,7 +364,8 @@ class Hatchet(object):
                 for rtype in self.typedb.relatedtypes(classnm,methnm):
                     for sclassnm in self.typedb.superclasses(rtype):
                         if sclassnm not in self.keep_classes:
-                            self.logger.debug("keeping class: %s",sclassnm)
+                            msg = "keeping class: %s [rtyp %s::%s]"
+                            self.logger.debug(msg,sclassnm,classnm,methnm)
                             self.keep_classes.add(sclassnm)
                             todo_classes.append(sclassnm)
 
@@ -373,28 +384,28 @@ class Hatchet(object):
             if methnm in kept_methods:
                 continue
             if methnm in used_ids:
-                self.logger.debug("keeping method: %s::%s",classnm,methnm)
+                self.logger.debug("keeping method: %s::%s [used]",classnm,methnm)
                 kept_methods.add(methnm)
             elif methnm + "_" in used_ids:
-                self.logger.debug("keeping method: %s::%s",classnm,methnm)
+                self.logger.debug("keeping method: %s::%s [used]",classnm,methnm)
                 kept_methods.add(methnm)
             elif methnm == classnm:
-                self.logger.debug("keeping method: %s::%s",classnm,methnm)
+                self.logger.debug("keeping method: %s::%s [cons]",classnm,methnm)
                 kept_methods.add(methnm)
             elif "*" in kept_methods:
-                self.logger.debug("keeping method: %s::%s",classnm,methnm)
+                self.logger.debug("keeping method: %s::%s [star]",classnm,methnm)
                 kept_methods.add(methnm)
             elif methnm in self.keep_methods.get("*",()):
-                self.logger.debug("keeping method: %s::%s",classnm,methnm)
+                self.logger.debug("keeping method: %s::%s [star]",classnm,methnm)
                 kept_methods.add(methnm)
             elif methnm in KEEP_METHODS.get(classnm,()):
-                self.logger.debug("keeping method: %s::%s",classnm,methnm)
+                self.logger.debug("keeping method: %s::%s [pinned]",classnm,methnm)
                 kept_methods.add(methnm)
             elif "*" in KEEP_METHODS.get(classnm,()):
-                self.logger.debug("keeping method: %s::%s",classnm,methnm)
+                self.logger.debug("keeping method: %s::%s [pinned]",classnm,methnm)
                 kept_methods.add(methnm)
             elif methnm in KEEP_METHODS.get("*",()):
-                self.logger.debug("keeping method: %s::%s",classnm,methnm)
+                self.logger.debug("keeping method: %s::%s [pinned]",classnm,methnm)
                 kept_methods.add(methnm)
             else:
                 #  Shiboken doesn't like it when we reject methods
@@ -403,7 +414,7 @@ class Hatchet(object):
                 #  TODO: is this just superstition on my part?
                 for sclassnm in self.typedb.superclasses(classnm):
                     if self.typedb.ispurevirtual(sclassnm,methnm):
-                        self.logger.debug("keeping method: %s::%s",classnm,
+                        self.logger.debug("keeping method: %s::%s [virt]",classnm,
                                                                    methnm)
                         kept_methods.add(methnm)
                         break
@@ -460,7 +471,7 @@ class Hatchet(object):
         used as a fallback location.
         """
         cachedir = get_cache_dir("Hatchet","src")
-        nm = os.path.basename(urlparse.urlparse(PYSIDE_SOURCE_URL).path)
+        nm = os.path.basename(urlparse.urlparse(self.SOURCE_URL).path)
         if cachedir is None:
             (fd,cachefile) = tempfile.mkstemp()
             os.close(fd)
@@ -468,40 +479,39 @@ class Hatchet(object):
             #  Use cached version if it has correct md5.
             cachefile = os.path.join(cachedir,nm)
             if os.path.exists(cachefile):
-                md5 = hashlib.md5()
-                with open(cachefile,"rb") as f:
-                    data = f.read(1024*32)
-                    while data:
-                        md5.update(data)
-                        data = f.read(1024*32)
-                if md5.hexdigest() != PYSIDE_SOURCE_MD5:
-                    self.logger.warn("bad MD5 for %r",cachefile)
-                    self.logger.warn("    %s != %s",md5.hexdigest(),
-                                                    PYSIDE_SOURCE_MD5)
+                if not self._check_pyside_source_md5(cachefile):
                     os.unlink(cachefile)
         #  Download if we can't use the cached version
         if cachedir is None or not os.path.exists(cachefile):
-            self.logger.info("downloading %s",PYSIDE_SOURCE_URL)
-            fIn = urllib2.urlopen(PYSIDE_SOURCE_URL)
+            self.logger.info("downloading %s",self.SOURCE_URL)
+            fIn = urllib2.urlopen(self.SOURCE_URL)
             try:
                  with open(cachefile,"wb") as fOut:
                     shutil.copyfileobj(fIn,fOut)
             finally:
                 fIn.close()
-            #  Verify the download
-            md5 = hashlib.md5()
-            with open(cachefile,"rb") as f:
-                data = f.read(1024*32)
-                while data:
-                    md5.update(data)
-                    data = f.read(1024*32)
-            if md5.hexdigest() != PYSIDE_SOURCE_MD5:
-                self.logger.critical("bad MD5 for %r",cachefile)
-                self.logger.critical("    %s != %s",md5.hexdigest(),
-                                                    PYSIDE_SOURCE_MD5)
-                raise RuntimeError("corrupted download: %s" % (url,))
+            if not self._check_pyside_source_md5(cachefile):
+                msg = "corrupted download: %s" % (PYSIDE_SOURCE_URL,)
+                raise RuntimeError(msg)
         return cachefile
 
+    def _check_pyside_source_md5(self,cachefile):
+        """Check the MD5 of a downloaded source file."""
+        if self.SOURCE_MD5 is None:
+            return True
+        md5 = hashlib.md5()
+        with open(cachefile,"rb") as f:
+            data = f.read(1024*32)
+            while data:
+                md5.update(data)
+                data = f.read(1024*32)
+        if md5.hexdigest() != self.SOURCE_MD5:
+            self.logger.critical("bad MD5 for %r",cachefile)
+            self.logger.critical("    %s != %s",md5.hexdigest(),
+                                                self.SOURCE_MD5)
+            return False
+        return True
+
     def unpack_tarball(self,sourcefile,destdir):
         """Unpack the given tarball into the given directory.
 
@@ -556,7 +566,8 @@ class Hatchet(object):
                logger.debug("reject %s::%s",rej[0],rej[1])
                num_rejected_methods += 1
                reject_methods.setdefault(rej[0],set()).add(rej[1])
-        logger.info("rejecting %s classes, %d methods",len(reject_classes),
+        logger.info("keeping %d classes",len(self.keep_classes))
+        logger.info("rejecting %d classes, %d methods",len(reject_classes),
                                                       num_rejected_methods)
         #  Find each top-level module directory and patch the contained files.
         psdir = os.path.join(sourcedir,"PySide")
@@ -740,19 +751,20 @@ class Hatchet(object):
             cxx = sysconfig.get_config_var("CXX")
             if cxx is not None:
                 env.setdefault("CXX",cxx)
-            cxxflags = sysconfig.get_config_var("CFLAGS") or ""
-            cxxflags += " " + env.get("CXXFLAGS","")
+            cxxflags = env.get("CXXFLAGS","")
+            cxxflags += " " + (sysconfig.get_config_var("CFLAGS") or "")
             if sys.platform != "win32":
                 cxxflags += " -fno-exceptions"
             if "linux" in sys.platform:
                 cxxflags += " -Wl,--gc-sections"
             env["CXXFLAGS"] = cxxflags
-            #  These are required for static linking on linux
-            #if sys.platform not in ("win32","darwin",):
-            #    ldflags = sysconfig.get_config_var("LDFLAGS")
-            #    ldflags += " " + env.get("LDFLAGS","")
-            #    ldflags += " --gc-sections -lpthread -lrt -lz -ldl lQtNetwork -lQtCore -ljpeg -ltiff -lpng14 -lz -lX11 -lXrender -lXrandr -lXext -lfontconfig -lSM -lICE"
-            #    env["LDFLAGS"] = ldflags
+            if "linux" in sys.platform:
+                ldflags = env.get("LDFLAGS","")
+                ldflags += " " + sysconfig.get_config_var("LDFLAGS")
+                #  These are required for static linking on linux
+                ldflags += " -lpthread -lrt -lz -ldl -lQtNetwork -lQtCore -ljpeg -ltiff -lpng14 -lz -lX11 -lXrender -lXrandr -lXext -lfontconfig -lSM -lICE"
+                ldflags += " --gc-sections"
+                env["LDFLAGS"] = ldflags
             cmd = ["cmake",
                    "-DCMAKE_BUILD_TYPE=MinSizeRel",
                    "-DCMAKE_VERBOSE_MAKEFILE=ON",
@@ -799,8 +811,12 @@ class Hatchet(object):
                         if filenm.endswith(modnm):
                             newfilepath = os.path.join(psdir,modnm)
                             break
-                #  If it's the pyside, replace that as well
-                elif "pyside." in filenm:
+                #  If it's the pyside support lib, replace that as well
+                elif filenm.startswith("libpyside"):
+                    newfilepath = os.path.join(sourcedir,"libpyside",filenm)
+                    if not os.path.exists(newfilepath):
+                        newfilepath = None
+                elif filenm.startswith("pyside") and filenm.endswith(".dll"):
                     newfilepath = os.path.join(sourcedir,"libpyside",filenm)
                     if not os.path.exists(newfilepath):
                         newfilepath = None
@@ -820,7 +836,6 @@ class Hatchet(object):
                 if newfilepath is not None:
                     self.copy_linker_paths(filepath,newfilepath)
                     self.logger.info("copying %r => %r",newfilepath,filepath)
-                    print "COPYING %r => %r" % (newfilepath,filepath,)
                     os.unlink(filepath)
                     shutil.copy2(newfilepath,filepath)
                     if "linux" in sys.platform:
@@ -828,6 +843,11 @@ class Hatchet(object):
                             _do("strip",filepath)
                         except subprocess.CalledProcessError:
                             pass
+                    elif sys.platform == "darwin":
+                        try:
+                            _do("strip","-S","-x",filepath)
+                        except subprocess.CalledProcessError:
+                            pass
 
     if sys.platform == "darwin":
         def copy_linker_paths(self,srcfile,dstfile):
@@ -1287,16 +1307,20 @@ if __name__ == "__main__":
     usage = "usage: Hatchet [options] /path/to/frozen/app [extra files]"
     op = optparse.OptionParser(usage=usage)
     op.add_option("-d","--debug",default="DEBUG",
-                  help="set the loggin debug level")
+                  help="set the logging debug level")
     op.add_option("","--follow-imports",
                   action="store_true",
                   dest="follow_imports",
                   help="follow import when loading code",
                   default=True)
     op.add_option("","--no-follow-imports",
-                  action="store_true",
+                  action="store_false",
                   help="don't follow imports when loading code",
                   dest="follow_imports")
+    op.add_option("","--analyse-only",
+                  action="store_true",
+                  help="just analyse the code, don't hack it",
+                  dest="analyse_only")
     (opts,args) = op.parse_args()
     try:
         opts.debugs = int(opts.debug)
@@ -1317,11 +1341,29 @@ if __name__ == "__main__":
     for fnm in args[1:]:
         if os.path.isdir(fnm):
             h.add_directory(fnm,follow_imports=opts.follow_imports)
-        if fnm.endswith(".zip") or nm.endswith(".exe"):
+        if fnm.endswith(".zip") or fnm.endswith(".exe"):
             h.add_zipfile(fnm,follow_imports=opts.follow_imports)
         else:
             h.add_file(fnm,follow_imports=opts.follow_imports)
-    h.hack()
+    if not opts.analyse_only:
+        h.hack()
+    else:
+        logger = logging.getLogger("PySideKick.Hatchet")
+        if not h.mf.modules:
+            h.add_directory(h.appdir)
+        num_rejected_classes = 0
+        num_rejected_methods = 0
+        for rej in h.find_rejections():
+            if len(rej) == 1:
+               logger.debug("reject %s",rej[0])
+               num_rejected_classes += 1
+            else:
+               logger.debug("reject %s::%s",rej[0],rej[1])
+               num_rejected_methods += 1
+        logger.info("keeping %d classes",len(h.keep_classes))
+        logger.info("rejecting %d classes, %d methods",num_rejected_classes,
+                                                       num_rejected_methods)
+
     sys.exit(0)
 
 
